require 'net/http'
require 'rubygems'
require 'hpricot'

class Article
  def initialize
    @host = 'www.linux-magazin.de'
    @baseurl = '/heft_abo/ausgaben'
  end
   
  def load_article_page(address)
    http = createConnection
    resp, data = http.get(address)
    return data
  end
   
  def read_articles(year, month)
    articles = get_articles_recursive(get_address(year, month))
    return 'keine Artikel gefunden' if !articles || articles.length == 0
    return articles.collect { |article|
      { :text => extract_article_text(article), 
        :link => extract_article_link(article) }
    }
    rescue SocketError
      return 'Netzwerkfehler'
  end

  private
  def createConnection
    @connection ||= Net::HTTP.new(@host)
  end

  def parse_list html
    parser = Hpricot(html)
    next_page = parser.at("span.next a")["href"] if parser.at("span.next a")
    return :articles => parser.search("div.attribute-short a.ue1"),
           :next_page => next_page
  end

  def get_articles_recursive address
    return [] unless address
    html = load_article_page(address)
    results = parse_list(html)
    return results[:articles].concat(get_articles_recursive(results[:next_page]))
  end
  
  def get_address(year, month)
    sprintf("%s/%d/%02d", @baseurl, year, month)
  end

  def get_base
    "http://" + @host
  end
  
  def extract_article_text(article)
    article.inner_text.strip
  end
  
  def extract_article_link(article)
    get_base + article['href']
  end
end

if __FILE__ == $0
  article = Article.new
  results = article.read_articles(2006,1)
  if(results.is_a?(String))
    puts results
  else
    results.each { |result|
      puts "---\n" + result[:text]
      puts result[:link] + "\n" if result[:link]
    }
  end
end
