require 'test/unit'
require 'article'
require 'rubygems'
require 'flexmock'

class Article
  attr_accessor :mock
  def createConnection
    return @mock
  end
end

class TestArticle < Test::Unit::TestCase
  include FlexMock::TestCase

  def setup
    @article = Article.new
    @year, @month = 2006,1
    @url_path = '/heft_abo/ausgaben/2006/01'
    @offset = '/(offset)/10'
    @url = 'http://www.linux-magazin.de' + @url_path
    @html_failure = File.read('fixtures/artikel_fehler.html')
    @html_first_page = File.read('fixtures/artikel_liste.html')
    @html_last_page = File.read('fixtures/artikel_letzte_seite.html')
    @mock = flexmock("http")
  end
   
  def test_parse_articles_no_content
    result = @article.send(:parse_list, @html_failure)[:articles]
    assert_equal(0, result.length, 'falsche Anzahl')
  end
   
  def test_parse_articles
    result = @article.send(:parse_list, @html_last_page)[:articles]
    assert_equal(2, result.length, 'falsche Anzahl')
  end

  def test_load_article_page_network_error
    @mock.should_receive(:get).once.returns {
      raise SocketError, 'Netzwerkfehler'
    }
    @article.mock = @mock
    assert_raise(SocketError) { @article.load_article_page(@url) }
  end

  def test_get_address
    base = @article.send(:get_base)
    url = @article.send(:get_address, @year, @month)
    assert_equal(@url, base + url)
  end

  def test_read_all_articles
    @mock.should_receive(:get).once.with(@url_path).and_return([nil, @html_first_page])
    @mock.should_receive(:get).once.with(@url_path+@offset).and_return([nil, @html_last_page])
    @article.mock = @mock
    result = @article.read_articles(@year, @month)
    assert_equal('Ruby spricht viele Sprachen',result[1][:text])
    assert_equal(@url + '/ruby_spricht_viele_sprachen',result[1][:link])
    assert_equal('Brave GNU World',result[11][:text])
    assert_equal(@url + '/brave_gnu_world',result[11][:link])
    assert_equal(12, result.length, 'falsche Anzahl')
  end

  def test_read_articles_last_page
    @mock.should_receive(:get).once.with(@url_path).and_return([nil, @html_last_page])
    @article.mock = @mock
    result = @article.read_articles(@year, @month)
    assert_equal('Brave GNU World',result[1][:text])
    assert_equal(@url + '/brave_gnu_world',result[1][:link])
    assert_equal(2, result.length, 'falsche Anzahl')
  end
   
  def test_read_articles_no_content
    @mock.should_receive(:get).once.returns { [nil, @html_failure] }
    @article.mock = @mock
    assert_nothing_raised {
      assert_equal("keine Artikel gefunden", @article.read_articles(@year, @month))
    }
  end
 
  def test_read_articles_network_error
    @mock.should_receive(:get).once.returns {
      raise SocketError, 'Netzwerkfehler'
    }
    @article.mock = @mock
    assert_nothing_raised {
      assert_equal("Netzwerkfehler", @article.read_articles(@year, @month))
    }
  end
end